import os

from django.conf import settings
from django.test import TestCase, override_settings
from django.core.urlresolvers import reverse

from evoteusers.tests.utils import UserTestUtils


project_path = settings.PROJECT_DIR


class ProccessSAMLResponseTest(UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.login()
        self.url = reverse("ama:samlresponse")

    @override_settings(ROOT_URLCONF="contrib.ama.tests.urls")
    def test_response_process(self):
        """ """
        self.respdata = {}
        testr_path = os.path.join(
            project_path, 'contrib/ama/tests/samlresptest.txt')
        testr = open(testr_path).read()
        self.respdata['SAMLResponse'] = testr
        self.respdata['RelayState'] = ";/test_url/;/test_err_url/"
        response = self.client.post(self.url, follow=False, data=self.respdata)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, "/test_url/")

    @override_settings(ROOT_URLCONF="contrib.ama.tests.urls")
    def test_invalid_signature_response_process(self):
        """ """
        self.respdata = {}
        testr_path = os.path.join(
            project_path,
            'contrib/ama/tests/samltestresp_error_invalidsignature.txt')
        testr = open(testr_path).read()
        self.respdata['SAMLResponse'] = testr
        self.respdata['RelayState'] = ";/test_url/;/test_err_url/"
        response = self.client.post(self.url, follow=False, data=self.respdata)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, "/test_err_url/")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)
