import base64
import cryptography

from lxml import etree
from signxml import XMLVerifier

from django.conf import settings
from django.shortcuts import redirect
from django.views.generic import View
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required

from django.utils.encoding import smart_str


saml_confs = getattr(settings, 'SAML', '')


@method_decorator(login_required, name="dispatch")
@method_decorator(csrf_exempt, name="dispatch")
class ProcessSAMLResponse(View):
    success = False

    def post(self, request, *args, **kwargs):
        # Read relay state data
        relaystate = request.POST.get("RelayState", "")
        (sessid, succtarget, errtarget) = relaystate.split(';')

        # Read saml response
        bsamlresp = request.POST.get("SAMLResponse", "")
        samlresp = base64.b64decode(bsamlresp)
        signed_root = etree.fromstring(samlresp)

        # Verify data Signature
        cert_path = saml_confs.get('idp_cert_path')
        cert = open(cert_path).read()
        if(cert):

            try:
                # signed_xml
                assertion_data =\
                    XMLVerifier().\
                    verify(signed_root, x509_cert=cert).signed_xml
            except cryptography.exceptions.InvalidSignature:
                pass

            else:
                # Reset auth gov session data
                request.session['authgov'] = {}
                # Save session id from the auth request
                request.session['authgov']['sessid'] = sessid

                # Read information from signed data
                samlns = "urn:oasis:names:tc:SAML:2.0:assertion"
                samlreqattributes = saml_confs.get('request_attributes')

                for attrid, attrname in samlreqattributes.items():
                    res = assertion_data.xpath(
                        '//fa:Attribute[@Name="{}"]'.format(attrname),
                        namespaces={'fa': samlns})
                    if (res and len(res)) > 0:
                        attr = res[0]
                        if len(attr) > 0:
                            txt = attr[0].text
                            val = smart_str(txt)
                            request.session['authgov'][attrid] = val
                            self.success = True

        if(self.success):
            return redirect(succtarget)
        else:
            return redirect(errtarget)
